@@ -11,6 +11,7 @@ from warnings import warn
 
 from .const import ERROR_CODES, MAX_LOGIN_RETRIES
 from .media_stream.session import HttpMediaSession
+from datetime import datetime
 
 urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
 
@@ -107,7 +108,7 @@ class Tapo:
         except Exception as e:
             raise Exception("Unexpected response from Tapo Camera: " + str(e))
 
-    def executeFunction(self, method, params):
+    def executeFunction(self, method, params, retry=False):
         if method == "multipleRequest":
             data = self.performRequest({"method": "multipleRequest", "params": params})[
                 "result"
@@ -123,9 +124,15 @@ class Tapo:
         if type(data) == list:
             return data
 
-        if "result" in data:
+        if "result" in data and (
+            "error_code" not in data
+            or ("error_code" in data and data["error_code"] == 0)
+        ):
             return data["result"]
         else:
+            if "error_code" in data and data["error_code"] == -64303 and retry is False:
+                self.setCruise(False)
+                return self.executeFunction(method, params, True)
             raise Exception(
                 "Error: {}, Response: {}".format(
                     data["err_msg"]
@@ -306,16 +313,6 @@ class Tapo:
         )
         return data["cet"]["media_encrypt"]
 
-    def getMotionDetection(self):
-        return self.executeFunction(
-            "getDetectionConfig", {"motion_detection": {"name": ["motion_det"]}},
-        )["motion_detection"]["motion_det"]
-
-    def getPersonDetection(self):
-        return self.executeFunction(
-            "getPersonDetectionConfig", {"people_detection": {"name": ["detection"]}},
-        )["people_detection"]["detection"]
-
     def getAlarm(self):
         # ensure reverse compatibility, simulate the same response for children devices
         if self.childID:
@@ -499,7 +496,26 @@ class Tapo:
             )["result"]["responses"][0]["result"]["user_id"]
         return self.userID
 
-    def getRecordings(self, date):
+    def getRecordingsList(self, start_date="20000101", end_date=None):
+        if end_date is None:
+            end_date = datetime.today().strftime("%Y%m%d")
+        result = self.executeFunction(
+            "searchDateWithVideo",
+            {
+                "playback": {
+                    "search_year_utility": {
+                        "channel": [0],
+                        "end_date": end_date,
+                        "start_date": start_date,
+                    }
+                }
+            },
+        )
+        if "playback" not in result:
+            raise Exception("Video playback is not supported by this camera")
+        return result["playback"]["search_results"]
+
+    def getRecordings(self, date, start_index=0, end_index=999999999):
         result = self.executeFunction(
             "searchVideoOfDay",
             {
@@ -507,9 +523,9 @@ class Tapo:
                     "search_video_utility": {
                         "channel": 0,
                         "date": date,
-                        "end_index": 99,
+                        "end_index": end_index,
                         "id": self.getUserID(),
-                        "start_index": 0,
+                        "start_index": start_index,
                     }
                 }
             },
@@ -523,19 +539,36 @@ class Tapo:
         warn("Prefer to use a specific value getter", DeprecationWarning, stacklevel=2)
         return self.performRequest({"method": "get", "image": {"name": "common"}})
 
+    def __getSensitivityNumber(self, sensitivity):
+        if sensitivity.isnumeric():
+            sensitivityInt = int(sensitivity)
+            if sensitivityInt >= 0 and sensitivityInt <= 100:
+                return str(sensitivityInt)
+            else:
+                raise Exception("Invalid sensitivity, can be between 0 and 100.")
+        else:
+            if sensitivity == "high":
+                return "80"
+            elif sensitivity == "normal":
+                return "50"
+            elif sensitivity == "low":
+                return "20"
+            else:
+                raise Exception("Invalid sensitivity, can be low, normal or high")
+
+    def getMotionDetection(self):
+        return self.executeFunction(
+            "getDetectionConfig", {"motion_detection": {"name": ["motion_det"]}},
+        )["motion_detection"]["motion_det"]
+
     def setMotionDetection(self, enabled, sensitivity=False):
         data = {
             "motion_detection": {"motion_det": {"enabled": "on" if enabled else "off"}},
         }
         if sensitivity:
-            if sensitivity == "high":
-                data["motion_detection"]["motion_det"]["digital_sensitivity"] = "80"
-            elif sensitivity == "normal":
-                data["motion_detection"]["motion_det"]["digital_sensitivity"] = "50"
-            elif sensitivity == "low":
-                data["motion_detection"]["motion_det"]["digital_sensitivity"] = "20"
-            else:
-                raise Exception("Invalid sensitivity, can be low, normal or high")
+            data["motion_detection"]["motion_det"][
+                "digital_sensitivity"
+            ] = self.__getSensitivityNumber(sensitivity)
         # child devices always need digital_sensitivity setting
         if (
             self.childID
@@ -547,22 +580,138 @@ class Tapo:
             ]
         return self.executeFunction("setDetectionConfig", data)
 
+    def getPersonDetection(self):
+        return self.executeFunction(
+            "getPersonDetectionConfig", {"people_detection": {"name": ["detection"]}},
+        )["people_detection"]["detection"]
+
     def setPersonDetection(self, enabled, sensitivity=False):
         data = {
             "people_detection": {"detection": {"enabled": "on" if enabled else "off"}}
         }
         if sensitivity:
-            if sensitivity == "high":
-                data["people_detection"]["detection"]["sensitivity"] = "80"
-            elif sensitivity == "normal":
-                data["people_detection"]["detection"]["sensitivity"] = "50"
-            elif sensitivity == "low":
-                data["people_detection"]["detection"]["sensitivity"] = "20"
-            else:
-                raise Exception("Invalid sensitivity, can be low, normal or high")
-
+            data["people_detection"]["detection"][
+                "sensitivity"
+            ] = self.__getSensitivityNumber(sensitivity)
         return self.executeFunction("setPersonDetectionConfig", data)
 
+    def getVehicleDetection(self):
+        return self.executeFunction(
+            "getVehicleDetectionConfig", {"vehicle_detection": {"name": ["detection"]}},
+        )["vehicle_detection"]["detection"]
+
+    def setVehicleDetection(self, enabled, sensitivity=False):
+        data = {
+            "vehicle_detection": {"detection": {"enabled": "on" if enabled else "off"}}
+        }
+        if sensitivity:
+            data["vehicle_detection"]["detection"][
+                "sensitivity"
+            ] = self.__getSensitivityNumber(sensitivity)
+        return self.executeFunction("setVehicleDetectionConfig", data)
+
+    def getPetDetection(self):
+        return self.executeFunction(
+            "getPetDetectionConfig", {"pet_detection": {"name": ["detection"]}},
+        )["pet_detection"]["detection"]
+
+    def setPetDetection(self, enabled, sensitivity=False):
+        data = {"pet_detection": {"detection": {"enabled": "on" if enabled else "off"}}}
+        if sensitivity:
+            data["pet_detection"]["detection"][
+                "sensitivity"
+            ] = self.__getSensitivityNumber(sensitivity)
+
+        return self.executeFunction("setPetDetectionConfig", data)
+
+    def getBarkDetection(self):
+        return self.executeFunction(
+            "getBarkDetectionConfig", {"bark_detection": {"name": ["detection"]}},
+        )["bark_detection"]["detection"]
+
+    def getMeowDetection(self):
+        return self.executeFunction(
+            "getMeowDetectionConfig", {"meow_detection": {"name": ["detection"]}},
+        )["meow_detection"]["detection"]
+
+    def setBarkDetection(self, enabled, sensitivity=False):
+        data = {
+            "bark_detection": {"detection": {"enabled": "on" if enabled else "off"}}
+        }
+        if sensitivity:
+            data["bark_detection"]["detection"][
+                "sensitivity"
+            ] = self.__getSensitivityNumber(sensitivity)
+
+        return self.executeFunction("setBarkDetectionConfig", data)
+
+    def setMeowDetection(self, enabled, sensitivity=False):
+        data = {
+            "meow_detection": {"detection": {"enabled": "on" if enabled else "off"}}
+        }
+        if sensitivity:
+            data["meow_detection"]["detection"][
+                "sensitivity"
+            ] = self.__getSensitivityNumber(sensitivity)
+
+        return self.executeFunction("setMeowDetectionConfig", data)
+
+    def getGlassBreakDetection(self):
+        return self.executeFunction(
+            "getGlassDetectionConfig", {"glass_detection": {"name": ["detection"]}},
+        )["glass_detection"]["detection"]
+
+    def setGlassBreakDetection(self, enabled, sensitivity=False):
+        data = {
+            "glass_detection": {"detection": {"enabled": "on" if enabled else "off"}}
+        }
+        if sensitivity:
+            data["glass_detection"]["detection"][
+                "sensitivity"
+            ] = self.__getSensitivityNumber(sensitivity)
+
+        return self.executeFunction("setGlassDetectionConfig", data)
+
+    def getTamperDetection(self):
+        return self.executeFunction(
+            "getTamperDetectionConfig", {"tamper_detection": {"name": "tamper_det"}},
+        )["tamper_detection"]["tamper_det"]
+
+    def setTamperDetection(self, enabled, sensitivity=False):
+        data = {
+            "tamper_detection": {"tamper_det": {"enabled": "on" if enabled else "off"}}
+        }
+        if sensitivity:
+            if sensitivity not in ["high", "normal", "low"]:
+                raise Exception("Invalid sensitivity, can be low, normal or high")
+            if sensitivity == "normal":
+                sensitivity = "medium"
+            data["tamper_detection"]["tamper_det"]["sensitivity"] = sensitivity
+
+        return self.executeFunction("setTamperDetectionConfig", data)
+
+    def getBabyCryDetection(self):
+        return self.executeFunction(
+            "getBCDConfig", {"sound_detection": {"name": ["bcd"]}},
+        )["sound_detection"]["bcd"]
+
+    def getCruise(self):
+        data = self.executeFunction(
+            "getPatrolAction", {"patrol": {"get_patrol_action": {}}}
+        )
+        return data
+
+    def setBabyCryDetection(self, enabled, sensitivity=False):
+        data = {"sound_detection": {"bcd": {"enabled": "on" if enabled else "off"}}}
+        if sensitivity:
+            if sensitivity not in ["high", "normal", "low"]:
+                raise Exception("Invalid sensitivity, can be low, normal or high")
+            if sensitivity == "normal":
+                sensitivity = "medium"
+            data["sound_detection"]["bcd"]["sensitivity"] = sensitivity
+
+        return self.executeFunction("setBCDConfig", data)
+
     def setAutoTrackTarget(self, enabled):
         return self.executeFunction(
             "setTargetTrackConfig",
@@ -573,6 +722,16 @@ class Tapo:
             },
         )
 
+    def setCruise(self, enabled, coord=False):
+        if coord not in ["x", "y"] and coord is not False:
+            raise Exception("Invalid coord parameter. Can be 'x' or 'y'.")
+        if enabled and coord is not False:
+            return self.executeFunction(
+                "cruiseMove", {"motor": {"cruise": {"coord": coord}}},
+            )
+        else:
+            return self.executeFunction("cruiseStop", {"motor": {"cruise_stop": {}}},)
+
     def reboot(self):
         return self.executeFunction("rebootDevice", {"system": {"reboot": "null"}})
 
@@ -789,6 +948,34 @@ class Tapo:
                         "method": "getPersonDetectionConfig",
                         "params": {"people_detection": {"name": ["detection"]}},
                     },
+                    {
+                        "method": "getVehicleDetectionConfig",
+                        "params": {"vehicle_detection": {"name": ["detection"]}},
+                    },
+                    {
+                        "method": "getBCDConfig",
+                        "params": {"sound_detection": {"name": ["bcd"]}},
+                    },
+                    {
+                        "method": "getPetDetectionConfig",
+                        "params": {"pet_detection": {"name": ["detection"]}},
+                    },
+                    {
+                        "method": "getBarkDetectionConfig",
+                        "params": {"bark_detection": {"name": ["detection"]}},
+                    },
+                    {
+                        "method": "getMeowDetectionConfig",
+                        "params": {"meow_detection": {"name": ["detection"]}},
+                    },
+                    {
+                        "method": "getGlassDetectionConfig",
+                        "params": {"glass_detection": {"name": ["detection"]}},
+                    },
+                    {
+                        "method": "getTamperDetectionConfig",
+                        "params": {"tamper_detection": {"name": "tamper_det"}},
+                    },
                     {
                         "method": "getLensMaskConfig",
                         "params": {"lens_mask": {"name": ["lens_mask_info"]}},
