@@ -1,5 +1,7 @@
 ERROR_CODES = {
     "-40401": "Invalid stok value",
+    "-40210": "Function not supported",
+    "-64303": "Action cannot be done while camera is in patrol mode.",
     "-64324": "Privacy mode is ON, not able to execute",
     "-64302": "Preset ID not found",
     "-64321": "Preset ID was deleted so no longer exists",
